package evaHelper;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.Calendar;

import org.jdom.Document;
import org.jdom.Element;

import android.os.Environment;



/**
 * This class should be called inside the code of the application that is being evaluated
 * This creates the output log file
 * 
 * This is language specific. Depends on how it is called. As of the moment, we'll try
 * using Android as an example.
 * 
 * Reference: 
 * http://java.sun.com/developer/technicalArticles/xml/JavaTechandXML/
 * http://www.javaworld.com/javaworld/jw-07-2000/jw-0728-jdom2.html
 * @author Florence
 *
 */
public class EvaLogger {

	//XML SPECIFIC DATA-not used
	private static Element 		root = new Element("mainLog");
	private static Document 	doc = new Document (root);
	private static String 		elementTags[] = {	"screen",
											"componentName",
											"componentType",
											"action",
											"time" 
											//ADD HERE
											};
	
	//PLATFORM DATA
	private static String		platform = "";
	public static String 		ANDROID_PLATFORM = "androidPlatform";
	public static String 		DESKTOP_PLATFORM = "desktopPlatform";
	
	//FILE INFO DATA	
	private	static File 			logFile		= null;// = new File(logFilename);
	public 	static String		evaLogExt  = ".evaLog";
	
	//BASIC LOG INFO
	private static String screenName = "";
	private static String componentName = "";
	private static String componentType = "";
		
	
	/**
	 * I'm still thinking if this should be a private class, only called by logX() method.
	 * Creates the log file.
	 * 
	 * @param platform	e.g. EvaLogger.ANDROID_PLATFORM
	 * @param filename	any filename for the logfile
	 */
	private static void startLogging(String platform, String filename){
		EvaLogger.platform = platform;
		
		//create the file
		try{
			if( EvaLogger.platform.equals(EvaLogger.ANDROID_PLATFORM)){
				File root = Environment.getExternalStorageDirectory();
				
				//FTBDebug( "Can we write to root?" + root.canWrite());
				
				logFile = new File(root, filename );
				
				if( !logFile.exists() ){
					logFile.createNewFile();
				}
				
				FTBDebug("FILE CREATED:" + logFile.getAbsolutePath());
				//FTBDebug("FILE INFO (does file exist?):" + logFile.exists());
			}
			else{
				//TODO
			}
		}
		catch( Exception e ){
			FTBDebug(e.toString());
		}
	}
	

	/**
	 * These are cases where we want to force the system to create a new log file instead of using the existing file
	 * This should be invoked before the logAndroid method
	 */
	public static void startNewLogFile(String platform) throws EvaLoggerException{
	
		//unsupported platforms
		if( !platform.equals(ANDROID_PLATFORM) && !platform.equals(DESKTOP_PLATFORM)){
			throw new EvaLoggerException("Platform not supported. See EvaLogger Documentation for a list of supported platforms.");
		}
		
		startLogging(platform, EvaLogger.makeMeALogFilename());
	}
	
	/**
	 * Should be called by classes where we want to log information.
	 * 
	 * @param screen screen name  
	 * @param componentName name of the component, use the label if possible, so that it's easy to trace
	 * @param componentType type/class of the component (e.g. button, textfield)
	 * @param action what type of event occurred when using the component (e.g. click, type, etc)
	 * 	
	 */
	public static void log(String screen,
								String componentName,
								String componentType,
								String action) 
								throws EvaLoggerException {
		try{
			File root = Environment.getExternalStorageDirectory();
			
			if( root.canWrite() ){				
				
				if( logFile == null ){
					startLogging(EvaLogger.ANDROID_PLATFORM, EvaLogger.makeMeALogFilename());
					FTBDebug("LOGANDROID [created file]:" + logFile.getAbsolutePath());
				}
				
				FileWriter logWriter = new FileWriter(logFile, true);//append
				BufferedWriter out = new BufferedWriter(logWriter);

				//let's make it comma separated for now
				String logMsg = screen +","+ componentName +","+ componentType +","+ action +","+ getLogTime() + "\n";
				
				out.write(logMsg);
				//out.flush();
				out.close();
				
				//FTBDebug("LOGANDROID:" + logMsg);
			}			
		
		}
		catch( Exception e ){
			FTBDebug(e.toString());
		}
	}	
	
	/**
	 * Should be called by classes where we want to log information.
	 *  
	 * @param componentName name of the component, use the label if possible, so that it's easy to trace
	 * @param componentType type/class of the component (e.g. button, textfield)
	 * @param action what type of event occurred when using the component (e.g. click, type, etc)
	 * 	
	 */
	public static void log(String componentName,
								String componentType,
								String action)
								throws EvaLoggerException {
		try{
			File root = Environment.getExternalStorageDirectory();
			
			if( root.canWrite() ){				
				
				if( logFile == null ){
					startLogging(EvaLogger.ANDROID_PLATFORM, EvaLogger.makeMeALogFilename());
					FTBDebug("LOGANDROID [created file]:" + logFile.getAbsolutePath());
				}
				
				FileWriter logWriter = new FileWriter(logFile, true);//append
				BufferedWriter out = new BufferedWriter(logWriter);

				//let's make it comma separated for now
				String logMsg = EvaLogger.screenName +","+ componentName +","+ componentType +","+ action +","+ getLogTime() + "\n";
				
				EvaLogger.FTBDebug("[WRITTEN TO FILE]" + logMsg);//Debug:TO REMOVE LATER
				
				out.write(logMsg);
				//out.flush();
				out.close();
				
				//FTBDebug("LOGANDROID:" + logMsg);
			}			
		
		}
		catch( Exception e ){
			FTBDebug(e.toString());
		}
	}	
	
	/**
	 * Should be called by classes where we want to log information.
	 * 
	 * @param screen screen name  
	 * @param componentName name of the component, use the label if possible, so that it's easy to trace
	 * @param componentType type/class of the component (e.g. button, textfield)
	 * @param action what type of event occurred when using the component (e.g. click, type, etc)
	 * 	
	 */
	public static void log(String action)throws EvaLoggerException {
		try{
			File root = Environment.getExternalStorageDirectory();
			
			if( root.canWrite() ){				
				
				if( logFile == null ){
					startLogging(EvaLogger.ANDROID_PLATFORM, EvaLogger.makeMeALogFilename());
					FTBDebug("LOGANDROID [created file]:" + logFile.getAbsolutePath());
				}
				else{
					FTBDebug("LOGANDROID [writing to file]:" +logFile.getAbsolutePath());
				}
				
				FileWriter logWriter = new FileWriter(logFile, true);//append
				BufferedWriter out = new BufferedWriter(logWriter);

				//let's make it comma separated for now
				String logMsg = EvaLogger.screenName +","+ EvaLogger.componentName +","+ EvaLogger.componentType +","+ action +","+ getLogTime() + "\n";
				
				out.write(logMsg);
				//out.flush();
				out.close();
				
				//FTBDebug("LOGANDROID:" + logMsg);
			}			
		
		}
		catch( Exception e ){
			FTBDebug(e.toString());
		}
	}	
	
	/**
	 * Log time
	 * @return logtime in format HH:MM:SS
	 */
	private static String getLogTime(){
		
		Calendar cal = Calendar.getInstance();
		
		String hour = cal.get(Calendar.HOUR_OF_DAY)>=10?(""+cal.get(Calendar.HOUR_OF_DAY)): ("0"+cal.get(Calendar.HOUR_OF_DAY)) ;
		String min = cal.get(Calendar.MINUTE)>=10?(""+cal.get(Calendar.MINUTE)): ("0"+cal.get(Calendar.MINUTE)) ;
		String sec = cal.get(Calendar.SECOND)>=10?(""+cal.get(Calendar.SECOND)): ("0"+cal.get(Calendar.SECOND)) ;
				
		String logTime = hour + ":" + min + ":" + sec;
		
		return logTime;		
	}
	
	/**
	 * Automatically creates a log filename
	 * @return a string filename
	 */
	private static String makeMeALogFilename(){
		Calendar cal = Calendar.getInstance();
		
		String hour = cal.get(Calendar.HOUR_OF_DAY)>=10?(""+cal.get(Calendar.HOUR_OF_DAY)): ("0"+cal.get(Calendar.HOUR_OF_DAY)) ;
		String min = cal.get(Calendar.MINUTE)>=10?(""+cal.get(Calendar.MINUTE)): ("0"+cal.get(Calendar.MINUTE)) ;
		String sec = cal.get(Calendar.SECOND)>=10?(""+cal.get(Calendar.SECOND)): ("0"+cal.get(Calendar.SECOND)) ;
		
		return "EvaLoggerFile" + hour + min + sec + evaLogExt;
	}
	////////////////Set basic logging info	
//	/**
//	 * 
//	 * @param screen
//	 * @param componentName
//	 * @param componentType
//	 */
//	public static void setBasicLogInfo( String screen, String componentName, String componentType ){
//		EvaLogger.screenName = screen.toString();
//		EvaLogger.componentName = componentName.toString();
//		EvaLogger.componentType = componentType.toString();
//	}	
//	
	/**
	 * 
	 * @param screen
	 */
	public static void setBasicLogInfo( String screen ){
		EvaLogger.screenName = screen.toString();		
	}		
	
//	/**
//	 * 
//	 * @param componentType
//	 */
//	public static void setBasicLogInfo( String componentName, String componentType ){		
//		EvaLogger.componentName = componentName.toString();
//		EvaLogger.componentType = componentType.toString();
//	}	
	
	////////////////My debugging methods
	/**
	 * My debug messages
	 * @param msg
	 */
	public static void FTBDebug(String msg){
		System.out.println("[FTBDebug]" + msg);
		
	}
	
	/**
	 * Lazy printing
	 * @param o
	 */
	public static void p(Object o){
		System.out.print(o);
	}
	
	/**
	 * Lazy printing
	 * @param o
	 */
	public static void pln(Object o){
		System.out.println(o);
	}
}
