package evaHelper;
import java.io.File;
import java.io.FileWriter;
import java.util.List;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;


/**
 * This class is used for parsing XML files and outputting them in specific formats
 * 
 * Usage Scenario:
 * 1. XML file with the log info
 * 2. EvaWriter reads the XML file and output it to CSV, GraphML or other format that is convenient for the evaluator
 * 
 * Reference: http://java.sun.com/developer/technicalArticles/xml/JavaTechandXML/
 * @author Florence
 */
public class EvaWriter {
	
	/**
	 * Reads an XML file and prints the data to screen
	 * @param filename
	 */
	public static void printXMLDataToScreen(String xmlFile){
		
		SAXBuilder builder = new SAXBuilder();
		
		try{
			Document	document = builder.build(xmlFile);
			
			Element 	root = document.getRootElement();
			List		rootChildrenList = root.getChildren();

			for( int i=0; i<rootChildrenList.size(); i++ ){
				
				Element log = (Element)rootChildrenList.get(i);
				pln(log.getName());
				
				List	logInfoList = log.getChildren();
								
				for(int j=0; j<logInfoList.size(); j++){
					
					Element logInfo = (Element)logInfoList.get(j);
					p("	" + logInfo.getName());
					pln(" = " + logInfo.getTextTrim());
				}
			}			
		}
		catch( Exception e ){
			FTBDebug( e.toString() );
		}		
	}
	
	/**
	 * Reads an XML file and outputs the data as a CSV file
	 * 
	 * TAGS - output as header
	 * DATA - output as elements in each row
	 * 
	 * @param xmlFile
	 * @param outputCSVFile
	 */
	public static void printXMLDataAsCSVFile( String xmlFile, String outputCSVFile ){
		SAXBuilder builder = new SAXBuilder();
		
		try{
			FileWriter fOut = new FileWriter(new File(outputCSVFile)); 			
			boolean	printHeadersDone = false;
			
			Document	document = builder.build(xmlFile);
			
			Element 	root = document.getRootElement();
			List		rootChildrenList = root.getChildren();		
			
			
			for( int i=0; i<rootChildrenList.size(); i++ ){
				
				Element log = (Element)rootChildrenList.get(i);
				List	logInfoList = log.getChildren();

				if(!printHeadersDone){
					for(int j=0; j<logInfoList.size(); j++){
						Element logInfo = (Element)logInfoList.get(j);
						String outStr = logInfo.getName() + (j==(logInfoList.size()-1)?"\n":",");
						fOut.write(outStr);						
						p(outStr);//debug
						
					}
					printHeadersDone=true;
				}
				
				for(int j=0; j<logInfoList.size(); j++){
					Element logInfo = (Element)logInfoList.get(j);
					String outStr = logInfo.getTextTrim() + (j==(logInfoList.size()-1)?"\n":",");
					fOut.write(outStr);						
					p(outStr);//debug
				}
			}//end for i
		
			fOut.close();			
		}
		catch( Exception e ){
			FTBDebug( e.toString() );
		}								
	}
	
	/**
	 * Reads a CSV file and outputs the data as a GraphML file
	 * 
	 * which are the nodes?
	 * lines connecting vary in length depending on time length?
	 * 
	 * @param xmlFile
	 * @param graphMLFile
	 */
	public static File printCSVDataAsGraphMLToFile(String csvFile, String outputGraphMLFile){
		//TODO: how to format in graphml (test output by showing it in YED)
		
		EvaGraphML evaGraph = new EvaGraphML(csvFile);
		File outputFile = null;
		
		try{
			outputFile = new File(outputGraphMLFile);
			FileWriter fOut = new FileWriter(outputFile);
			fOut.write(evaGraph.getCreatedGraphMLString());
			fOut.close();
		}
		catch( Exception e ){
			FTBDebug(e.toString());
		}
		
		return outputFile;
	}
	
	private void Test(){
		SAXBuilder builder = new SAXBuilder();
		
	}
	
	//////////////GRAPHML INFO////////////
/*  http://graphml.graphdrawing.org/primer/graphml-primer.html

	BASIC:
	<?xml version="1.0" encoding="UTF-8"?>
	<graphml xmlns="http://graphml.graphdrawing.org/xmlns">
	  ...
	</graphml>
	
	NODE:	
	<node id="n0"/> ->id must be unique
	
	EDGE:
	<edge id="e1" directed="true" source="n0" target="n2"/>
	
	NESTED GRAPHS:
	<?xml version="1.0" encoding="UTF-8"?>
	<graphml xmlns="http://graphml.graphdrawing.org/xmlns"  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
	xsi:schemaLocation="http://graphml.graphdrawing.org/xmlns http://graphml.graphdrawing.org/xmlns/1.0/graphml.xsd">
	  <graph id="G" edgedefault="undirected">
		 ...
		 <node id="n6">
	        <graph id="n6:" edgedefault="undirected">
	          <node id="n6::n0">
	              <graph id="n6::n0:" edgedefault="undirected">
	                <node id="n6::n0::n0"/>
	               </graph>
	          </node>
	          <node id="n6::n1"/>
	          <node id="n6::n2"/>
	          <edge id="e10" source="n6::n1" target="n6::n0::n0"/>
	          <edge id="e11" source="n6::n1" target="n6::n2"/>
	        </graph>
	    </node>
       </graph>
    </graphml>
	 ...
*/
	////////////////My debugging methods
	/**
	 * My debug messages
	 * @param msg
	 */
	public static void FTBDebug(String msg){
		System.out.println("[FTBDebug]" + msg);
	}

	/**
	 * Lazy printing
	 * @param o
	 */
	public static void p(Object o){
		System.out.print(o);
	}
	
	/**
	 * Lazy printing
	 * @param o
	 */
	public static void pln(Object o){
		System.out.println(o);
	}
}
